import pyglet
import math
import core


class Collisions:
    def __init__(self, window, spritelist):
        self.window = window
        self.test = None
        self.spritelist = spritelist
        self.collisions = set()
        self.drag = False
        self.yarns = []
        self.yarn_im = core.lib.image.load_resource("yarn.png", "center")
        window.push_handlers(on_mouse_motion=self._mouse_motion, on_mouse_press=self._mouse_press,
                             on_mouse_drag=self._mouse_drag, on_mouse_release=self._mouse_release)


    def _mouse_motion(self, x, y, dx, dy):
        self.detect_collision(x, y, "m_hover")

    def _mouse_press(self, x, y, button, modifiers):
        print("press", x, y, button)
        self.x = x * self.window.board._scale - self.window.board._r_x
        self.y = y * self.window.board._scale - self.window.board._r_y

    def _mouse_release(self, x, y, button, modifiers):
        print("release", x, y, button)
        del self.test
        self.test = None
        for i in self.yarns:
            del i

    def _mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        self.drag = True
        dx = x - self.x
        dy = y - self.y
        angle = math.degrees(math.atan2(dx, dy))
        # print("drag", dx, dy, angle)
        end_x = x * self.window.board._scale - self.window.board._r_x
        end_y = y * self.window.board._scale - self.window.board._r_y

        for i in self.yarns:
            del i
        cx = self.x
        cy = self.y
        print(math.ceil(math.sqrt(dx ** 2 + dy ** 2) / 1024*.025))
        for i in range(math.ceil(math.sqrt(end_x ** 2 + end_y ** 2) / 1024*.025)):
            asdf = core.lib.board.BoardSprite(self.window.board, self.yarn_im, cx, cy, scale=.025, group=self.window.board.yarn_group)
            asdf.rotation = angle
            self.yarns.append(asdf)
            cx = cx + (1024*.025) * math.cos(math.radians(angle))
            cy = cy + (1024*.025) * math.sin(math.radians(angle))


    def detect_collision(self, x, y, type):
        for sprite in self.spritelist:
            if not sprite._visible:
                self.collisions.remove(sprite)
                continue
            if sprite._x - sprite.width<= x and sprite._x >= x and \
                sprite._y <= y and sprite._y + sprite.height >= y:
                if sprite not in self.collisions:
                    # Collision detected
                    sprite.collision(type)
                    self.collisions.add(sprite)
            else:
                self.collisions.discard(sprite)
                # pass

